@@ -1,6 +1,6 @@
-#! /bin/ksh
+#!/bin/sh
 aclocal -I config
 libtoolize --force --copy
-autoheader
 automake --foreign --add-missing --copy
 autoconf
+
