@@ -1,15 +1,18 @@
 /*****************************************************************************/
 /*                                                                           */
-/* Copyright (c) 1990-2001 Morgan Stanley Dean Witter & Co. All rights reserved.*/
+/* Copyright (c) 1990-2008 Morgan Stanley All rights reserved.
+*/
 /* See .../src/LICENSE for terms of distribution.                           */
 /*                                                                           */
 /*                                                                           */
-/*****************************************************************************/#include <stdio.h>
+/*****************************************************************************/
+#include <stdio.h>
 #include <fcntl.h>
 #include <errno.h>
 #include <string.h>
 #include <syslog.h>
 #include <sys/types.h>
+#include <unistd.h>
 #include <sys/stat.h>
 #include <sys/mman.h>
 #include <sys/param.h>
@@ -31,6 +34,8 @@ extern C *findMapped32FileName();
 extern C *findMapped64FileName();
 extern C *findFileName();
 
+Z I items(I n,A z);
+
 /* flag to control auto/endain conversion of the .m file  */
 Z I autoBeamConvert=0;		/* Default is off  */
 I getAutoBeamConvert(void)   {R autoBeamConvert;}
@@ -53,7 +58,7 @@ static C *endianString(I endian)
     (endian==ENDIAN_UNDEF)?"undef":"unknown";
 }
 
-static int hostEndian(void)
+static I hostEndian(void)
 {
   return (0x61626364==*EndianTag.u)?ENDIAN_BIG:
     (0x64636261==*EndianTag.u)?ENDIAN_LITTLE:
@@ -118,7 +123,7 @@ static void ndnfswap(F *start,I nfs)
   }
 }
 
-static int aobjEndian(A aobj)
+static I aobjEndian(A aobj)
 {
   int result=ENDIAN_UNDEF;
   I tempint;
@@ -136,7 +141,7 @@ static int aobjEndian(A aobj)
 
 static void doSwapEndianInPlace(A aobj, I targetndn)  
 {
-  int n,t;
+  I n,t;
   I sourcendn=aobjEndian(aobj);
   I hostndn=hostEndian();
   if(targetndn==sourcendn)return;
@@ -159,7 +164,7 @@ static void doSwapEndianInPlace(A aobj, I targetndn)
   case Ct:
     break;
   default:
-    printf("\343 error!!: doSwapEndianInPlace encountered bad type:%d\n",t);
+    printf("\343 error!!: doSwapEndianInPlace encountered bad type:%ld\n",t);
     break;
   }
 }
@@ -245,7 +250,7 @@ A ep_SwapToEndian(A aobj,A andn) /* must use argument type 12, not 0!!! */
 
 Z int loopen(C *path,int flags,mode_t mode)
 {
-  I i=0,st=1,fd=-1;
+  int i=0,st=1,fd=-1;
   static C fcn[]="open";
   if(!path) R -1;
   while(i<10&&-1==(fd=open(path,flags,mode))&&EWOULDBLOCK==errno)
@@ -306,7 +311,7 @@ void mv32(I *dest64,int *src32,int n) {
 }
 
 A
-gc32(int t,int r,int n,int d[],int *p) {
+gc32(int t, int r, int n, int d[], int *p) {
   I f ;
   A z ;
 
@@ -329,7 +334,7 @@ gc32(int t,int r,int n,int d[],int *p) {
 A
 cvt64(A32 *aobj) {   /* mapped objects only - floats, chars and longs */
     A newa;
-	int i = aobj->i ;
+    I i = aobj->i ;
     /*printf("%d %d %d %d %d\n", aobj->c, aobj->t, aobj->r, aobj->n, aobj->i) ; */
     newa = gc32(aobj->t,aobj->r,aobj->n,aobj->d,aobj->p);
 	newa->i = i ;
@@ -337,7 +342,7 @@ cvt64(A32 *aobj) {   /* mapped objects only - floats, chars and longs */
 }
 
 
-Z A vetteMappedFile(I fsize,I aarg,int mode,C *name, I *rc, I cvtInPlace)
+Z A vetteMappedFile(I fsize, I aarg, int mode, C *name, int *rc, I cvtInPlace)
 {
   A aobj=(A)aarg,z;
 
@@ -361,7 +366,7 @@ Z C MFAErrorMsg[128];
 Z int MFArealloc(I newlim)
 {
   Z MFInfo *newMFA;
-  int i, count=0;
+  I i, count=0;
   if(0==newlim||newlim<maxMFAindex)
   {
     strcpy(MFAErrorMsg,"New limit is less than Mapped File Array length");
@@ -436,11 +441,13 @@ Z void unmapDotMFile(A a,MFInfo *p)
 {
   /* printf("In unmapDotMFile(%s, %lx, %ld)\n", p->t, a, p->n) ; */
   if(dbg_tb) beamtrc(p->t,2,0);
-  if(munmap((caddr_t)a,(int)p->n)) 
+  if(munmap((void*)a,(size_t)p->n)) 
   {
     I nern=errno;
+#ifndef __ia64
     syslog(LOG_INFO,"A+ munmap() failed for %s with args (%#lx,%ld) with %m",
-	   p->t,a,p->n);
+	   p->t,a,p->n,nern);
+#endif
     H("\343 A+ munmap() failed for %s with args (%#lx,%ld) and errno=%ld\n",
       p->t,a,p->n,nern);
   }
@@ -453,28 +460,33 @@ Z void unmapDotMFile(A a,MFInfo *p)
   if(p==MappedFileArray+(maxMFAindex-1))--maxMFAindex;
 }
 
-extern I map(int,int);
 Z I mapDotMFile(int fd,int mode,C *fname,C *t)
 {
   A aobj;
-  I mfile,rc;
+  I mfile; int rc;
   MFInfo *p;
-  I n;
-  (void) setStickyBit(fd,0);
+  off_t n;
   n=lseek(fd,0,SEEK_END);
   mfile=map(fd,mode);		/* map returns a 0 on failure    */
   Q(!mfile,9);			/* Set domain error if map fails */
   /* printf("mfile = %ld, n = %ld\n", mfile, n) ; */
   aobj=vetteMappedFile(n, mfile, mode, fname, &rc,
 		       mode==BEAM_RW && autoBeamConvert);
-  if(rc==-1) 
+  if(!aobj || rc==-1) 
     {
+      if(dbg_tb) beamtrc(t,2,0);
       munmap((caddr_t)mfile,n);
-/*       H("\343 Conversion error\n"); */
       R q=9,0;
     }
 
-  Q(!aobj,9);
+  if(rc==1 && mode==BEAM_RW && autoBeamConvert==0) 
+    {
+      if(dbg_tb) beamtrc(t,2,0);
+      munmap((caddr_t)mfile,n);
+      dc(aobj);
+      printf("\343 Error: read/write, but conversion to local variable\n");
+      R q=9,0;
+    }
 
   /* The following if clause will convert the .m file to the native format */
   /* for 32 bit to 64 bit otherwise it is done in place */
@@ -482,49 +494,53 @@ Z I mapDotMFile(int fd,int mode,C *fname,C *t)
   if(rc==1 && mode==BEAM_RW && autoBeamConvert==1) /* Convert .m file */
     {
       struct a a;
-      I itemCount=aobj->i;
-      if(dbg_tb) printf("\343 Converting %s\n",t);
-      p=getFreeMFInfoStruct();
-      if(!p)
-        {
-          R H("maplim\n"),dc(aobj),q=9,0;
-        }
-      p->c=1,p->n=n,p->w=mode,p->s=bstring(fname),p->t=bstring(t),p->a=mfile;
-      unmapDotMFile((A)mfile,p);
+      I itemCount=aobj->i;	/* original item count */
 
-#if (_MIPS_SZLONG == 64) || defined(__alpha) || defined(__sparcv9)
-      t = findMapped64FileName(fname, mode) ;
-      if (!t)
-        t = findMapped32FileName(fname, mode) ;
-#endif
+      if(dbg_tb) beamtrc(t,3,0); /* 3==Converting */
 
-      mapOut(t, aobj);
-      dc(aobj);
-      aobj=0;
+      if(dbg_tb) beamtrc(t,2,0);
+      munmap((caddr_t)mfile,n);	/* unmap previous */
+      mapOut(t, aobj);		/* write new .m with converted data */
+      dc(aobj);			/* free a object */
+                                /* reset the items to match orignal*/
+      {
+	A mFile=gsv(0,t);
+	I rc=items(itemCount, mFile);
+	if(dbg_tb) printf("\343 Setting Items to %ld\n",itemCount);
+	dc(mFile);
+      }
 
-      ERR(t, fd=loopen(t,O_RDWR,0666));
-      if(-1==read(fd,(C *)&a,AH))
-	return perror(t), close(fd), q=9, 0;
-      if(-1==flen(fd,AH+Tt(a.t,a.i)+(a.t==Ct)))
-	return perror(t), close(fd), q=9, 0;
-      if(-1==lseek(fd,0,SEEK_SET))
-	return perror(t), close(fd), q=9, 0;
+				/* re-map it */
+      ERR(t, fd=loopen(t,BEAM_RW==mode?O_RDWR:O_RDONLY,0666));
+      n=lseek(fd,0,SEEK_END);
+      mfile=map(fd,mode);	/* map returns a 0 on failure    */
+      Q(!mfile,9);		/* Set domain error if map fails */
 
-      setStickyBit(fd,0);
-      mfile=map(fd,mode);               /* map returns a 0 on failure    */
-      Q(!mfile,9);                      /* Set domain error if map fails */
-      if(dbg_tb)beamtrc(t,1,mode);
-      aobj=vetteMappedFile(n,mfile,mode,fname,&rc,0);
-      Q(!aobj,9);
+      /* Call  vetteMappedFile with autoBeamConvert off */
+      aobj=vetteMappedFile(n, mfile, mode, fname, &rc, 0);
+
+      if(!aobj || rc!=0)
+	{
+	  if(dbg_tb) beamtrc(t,2,0);
+	  munmap((caddr_t)mfile,n);
+	  R q=9,0;
+	}
     }
 
-  p=getFreeMFInfoStruct();
-  if(!p)
-  {
-    R H("maplim\n"),dc(aobj),q=9,0;
-  }
-    p->c=1,p->n=n,p->w=mode,p->s=bstring(fname),p->t=bstring(t),p->a=mfile;
-    if(aobj!=(A)mfile) unmapDotMFile((A)mfile,p);
+  if(rc==1)			/* Conversion to regular variable*/
+    {
+      if(dbg_tb) beamtrc(t,2,0);
+      munmap((caddr_t)mfile,n);	/* unmap previous */
+    }
+  else
+    {
+      p=getFreeMFInfoStruct();
+      if(!p)
+	{
+	  R H("maplim\n"),dc(aobj),q=9,0;
+	}
+      p->c=1,p->n=n,p->w=mode,p->s=bstring(fname),p->t=bstring(t),p->a=mfile;
+    }
   R (I)aobj;
 }
 
@@ -554,7 +570,7 @@ Z void dbg_mfapp(MFInfo *p)
 
 void dbg_mfa(void)
 {
-  int i;
+  I i;
   H("\343 maplim:%ld  maxIndex:%ld  freeIndex:%ld\n",maxMFAlimit,maxMFAindex,
     freeMFAindex);
   if(0==MappedFileArray){H("\343 Mapped File Array not initialized.\n");R;}
@@ -565,7 +581,7 @@ void dbg_mfa(void)
   for(i=maxMFAindex;i<maxMFAindex+10;i++)
   {
     if(i>=maxMFAlimit)break;
-    H("\343  %d: ",i);
+    H("\343  %ld: ",i);
     if(MappedFileArray[i].a)dbg_mfapp(MappedFileArray+i);
     else H("<free>  next:%ld\n",MappedFileArray[i].n);
   }
@@ -582,7 +598,7 @@ void dbg_mfr(void)
 
 A dbg_mfrsf(void)
 {
-  int n=0;
+  I n=0;
   A z,modes,args,fnames,addrs,refcnts,bytes;
   MFInfo *p;
   z=gv(Et,2);
@@ -613,12 +629,12 @@ A dbg_mfrsf(void)
 } 
 
 I mapIn(C *name,I mode){
-  I fd=-1,z;
-  I tmode = mode ;
+  int fd=-1; I z;
+  int tmode = mode ;		/* Valid modes 0, 1, or 2  */
   C*t=0;
   /* printf("In mapIn(%s, %d)\n", name, mode) ;*/
   Q(!name||BEAM_RO>mode||BEAM_LOCAL<mode,9);
-#if (_MIPS_SZLONG == 64) || defined(__alpha) || defined(__sparcv9)
+#if (_MIPS_SZLONG == 64) || defined(__alpha) || defined(__sparcv9) || defined(_ia64)
   /* First look for a 64 bit file */
   t = findMapped64FileName(name, (BEAM_RW==tmode)) ;
   if (!t)
@@ -634,31 +650,34 @@ I mapIn(C *name,I mode){
   R z;
 }
 
-Z writeAobjToFile(int fd,C *s,unsigned n,I c)
+Z ssize_t writeAobjToFile(int fd, C *s, size_t n, I c)
 {
-  I t, newItems;
+  ssize_t t; off_t lt; I newItems;
   A a=(A) s; 
   /* printf("in writeAobj - n = %u\n", n) ; */
   do t=write(fd,s,n); while(s+=t,t!=-1&&(n-=t));
   if((!c) && t!=-1 && a->i!=(newItems=a->r ? a->d[0] : a->n)) 
   { 
     /* if beaming out a mapped variable, writeover a->i with a->d[0]  */
-    t=lseek(fd,(AH-sizeof(I)),SEEK_SET);
-    if(t!=-1) t=write(fd,(C *)&newItems,sizeof(I));
+    lt=lseek(fd,(AH-sizeof(I)),SEEK_SET);
+    if(lt!=-1) t=write(fd,(C *)&newItems,sizeof(I));
   }
   fsync(fd);R t;
 } /* IBM write fix */
 
 Z I mapOut(C *s,A z)
 {
-  I fd,rc,c=0;
+  int fd; I rc,c=0;
   C r[MAXPATHLEN];
   C *dotPos, *slashPos;
-#if (_MIPS_SZLONG == 64) || defined(__alpha) || defined(__sparcv9)
-  static C defaultSuffix[]={"m64"} ;
-#else
+/* #if (_MIPS_SZLONG == 64) || defined(__alpha) || defined(__sparcv9) || defined(_ia64) */
+/*   static C defaultSuffix[]={"m64"} ; */
+/* #else */
+/*   static C defaultSuffix[]={"m"}; */
+/* #endif */
+
   static C defaultSuffix[]={"m"};
-#endif
+
   Q(Ct<z->t,6);
 
 /* On solaris this comparison was failing when '.' was zero and the address */
@@ -686,6 +705,7 @@ Z I mapOut(C *s,A z)
   if(z->c)c=z->c,z->c=0,z->i=z->r?*z->d:1;
   rc=writeAobjToFile(fd,(C *)z,AH+Tt(z->t,z->n)+(Ct==z->t?1:0),c);
   if(c)z->c=c;
+  fsync(fd);
   close(fd);
   ERR(s,rc);
   ERR(s,rename(r,s));
@@ -707,7 +727,7 @@ Z I setSizeOfFile(int fd,off_t n)
 #ifdef __VISUAL_C_2_0__
   I j=CLBYTES,k=lseek(fd,0,SEEK_END);
 #else
-  I j=getpagesize(),k=lseek(fd,0,SEEK_END);
+  size_t j=getpagesize(),k=lseek(fd,0,SEEK_END);
 #endif
   C junk[4];
   /* printf("In setSizeOfFile; n = %ld, j = %ld, k = %ld\n",
@@ -729,12 +749,12 @@ Z I setSizeOfFile(int fd,off_t n)
 
 Z I items(I n,A z)
 {
-  C *s;Z struct a a;I fd,t,m,j,w=n!=-1;
+  C *s;Z struct a a;int fd; I t,m,j,w=n!=-1;
   C *name ;
   Q(-2>n,9);
   NDC1(z);s=stringFromAobj(z);Q(!s,9);
 
-#if (_MIPS_SZLONG == 64) || defined(__alpha) || defined(__sparcv9)
+#if (_MIPS_SZLONG == 64) || defined(__alpha) || defined(__sparcv9) || defined(__ia64)
   name = findMapped64FileName(s, w) ;
   if (!name)
 	name = findMapped32FileName(s, w) ;
@@ -747,7 +767,7 @@ Z I items(I n,A z)
     return perror(s),close(fd),q=9,0;
 
   {
-    I ret, totsize,rank,itemCount,items;
+    int ret; I totsize,rank,itemCount,items;
     struct stat statbuf;
     if ( -1==fstat(fd, &statbuf) ) /* To get size for getItems() */
       return perror(s), close(fd), q=9, 0;
@@ -778,7 +798,7 @@ Z I items(I n,A z)
 	  return 0;
 	else
 	  dc(aobj);
-#if (_MIPS_SZLONG == 64) || defined(__alpha) || defined(__sparcv9)
+#if (_MIPS_SZLONG == 64) || defined(__alpha) || defined(__sparcv9) || defined(__ia64)
 	name = findMapped64FileName(s, w) ;
 	if (!name)
 	  name = findMapped32FileName(s, w) ;
@@ -823,7 +843,8 @@ static long ep_isaDotM(char *fname)
 {
   long rc=0;                    /* 0 == invalid dot M file */
   A aobj;
-  I mlen,fd;
+  off_t mlen;
+  int fd;
 
   if(-1==(fd=open(fname,O_RDONLY)))
     {
@@ -860,7 +881,7 @@ static long ep_isaDotM(char *fname)
 
 static void ep_msyncAll(A msyncMode_)
 {
-  int msyncFlag;
+  int msyncFlag=0;
   long i;
 
   if(msyncMode_->t!=Et || !QS(*msyncMode_->p) )
@@ -869,18 +890,41 @@ static void ep_msyncAll(A msyncMode_)
       R;
     }
 
-  if(*msyncMode_->p==MS(si("MS_ASYNC")))
+  for(i=0; i<msyncMode_->n; i++)
     {
-      msyncFlag=MS_ASYNC;
-    }
-  else if(*msyncMode_->p==MS(si("MS_SYNC")))
-    {
-      msyncFlag=MS_SYNC; 
-    }
-  else
-    {
-      q=ERR_DOMAIN;
-      R;
+      if(msyncMode_->p[i]==MS(si("MS_ASYNC")))
+	{
+          if(msyncFlag&MS_SYNC)
+	    {
+	      q=ERR_DOMAIN;
+	      return;
+	    }
+	  else
+	    {
+	      msyncFlag|=MS_ASYNC;
+	    }
+	}
+      else if(msyncMode_->p[i]==MS(si("MS_SYNC")))
+	{
+          if(msyncFlag&MS_ASYNC)
+	    {
+	      q=ERR_DOMAIN;
+	      return;
+	    }
+	  else
+	    {
+	      msyncFlag|=MS_SYNC;
+	    }
+	}
+      else if(msyncMode_->p[i]==MS(si("MS_INVALIDATE")))
+	{
+	  msyncFlag|=MS_INVALIDATE;
+	}
+      else
+	{
+	  q=ERR_DOMAIN;
+	  return;
+	}
     }
 
   for(i=0; i<maxMFAindex; i++)
@@ -888,7 +932,8 @@ static void ep_msyncAll(A msyncMode_)
       MFInfo *p=MappedFileArray+i;
       if(p->a  && p->w==1 )
 	{
-	  if(-1==msync((A)p->a, p->n, msyncFlag))
+	  A a=(A)p->a;
+	  if(-1==msync((C*)a, AH+Tt(a->t,a->n), msyncFlag))
 	    {
 	      printf("\343 Error: %s\n",p->t); 
 	      perror("ep_amsyncAll: msync");
