@@ -1,7 +1,17 @@
+/*****************************************************************************/
+/*                                                                           */
+/* Copyright (c) 1990-2008 Morgan Stanley All rights reserved.
+*/
+/* See .../src/LICENSE for terms of distribution.                           */
+/*                                                                           */
+/*                                                                           */
+/*****************************************************************************/
+
 /* This file contains routines that identify and convertthe different
  * formats of mmap'd files used by A+
  */
 
+#include <a/fncdcls.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
@@ -102,7 +112,7 @@ static void SwapEndianHeader32(A32 *header)
 		
 /* tr and tr32 -- from k.c */
 
-static INT64 tr64(INT64 r, INT64 *d)
+INT64 tr64(INT64 r, INT64 *d)
 {
   INT64 n=1;
   INT64 *t;
@@ -238,6 +248,45 @@ static int GetSrcInformation(void *src, I iBytesRead, I noSizeChk,
   */
 
 #if defined(linux)
+
+#if defined(__ia64) || defined(__x86_64)
+  /* Try 64-bit little endian */
+  /* Valid values: 0 <= type <= 8, 0 < rank <= 9, n > 0 */
+  if ( isAObject(a64LittleHeader) )
+    {
+       int dimsOK=1;
+       /* Check dimensions */
+       checkDims(a64LittleHeader);
+
+      /* Check file length */
+      if (dimsOK && 
+	  noSizeChk || (sizeOK=checkFileSize64(a64LittleHeader)<=iLen))
+	{
+	  *width = 64;
+	  *endian = ENDIAN_LITTLE;
+	  return 0;
+	}
+    }
+
+  /* Try 64-bit big endian */
+  /* Valid values: 0 <= type <= 8, 0 < rank <= 9, n > 0 */
+  if ( isAObject(a64BigHeader) )
+    {
+       int dimsOK=1;
+       /* Check dimensions */
+       checkDims(a64BigHeader);
+
+      /* Check file length */
+      if (dimsOK && 
+	  noSizeChk || (sizeOK=checkFileSize64(a64BigHeader)<=iLen))
+	{
+	  *width = 64;
+	  *endian = ENDIAN_BIG;
+	  return 0;
+	}
+    }
+#endif
+
   /* Try 32-bit little endian */
   /* Valid values: 0 <= type <= 8, 0 < rank <= 9, n > 0 */
   if ( isAObject(a32LittleHeader) )
@@ -276,7 +325,7 @@ static int GetSrcInformation(void *src, I iBytesRead, I noSizeChk,
     }
 		
 
-#elif (_MIPS_SZLONG == 64) || defined(__alpha) || defined(__sparcv9)
+#elif (_MIPS_SZLONG == 64) || defined(__alpha) || defined(__sparcv9) 
   /* Try 64-bit big endian */
   /* Valid values: 0 <= type <= 8, 0 < rank <= 9, n > 0 */
   if ( isAObject(a64BigHeader) )
@@ -467,7 +516,7 @@ static int Convert32to64(A32 *from, A64 *to)
 
   if(((void *)from) == ((void *)to) )
     {
-      printf("\343 Error:In place conversion not supported for 32->64 bit\n");
+      /* This conversion done in mapDotMFile() */
       return -1;
     }
 
@@ -475,6 +524,7 @@ static int Convert32to64(A32 *from, A64 *to)
   to->t = from->t;
   to->r = from->r;
   to->n = from->n;
+  to->i = from->i;
   for (i = 0; i < 9; i++)
     to->d[i] = from->d[i];
   switch(from->t)
@@ -576,7 +626,7 @@ static int ConvertEndian64(A64 *from, A64 *to)
 	break;
       }
     default: /* Bad type */
-      printf("\343 Error:Type %d not translatable.\n", from->t);
+      printf("\343 Error:Type %ld not translatable.\n", from->t);
       return -1;
     }
   return 0;
@@ -628,9 +678,9 @@ int cvtIfNeeded(void *src, A *dest, I ilen, I cvtInPlace)
       return 0;
     }
 
-  /* Case 2: Big/32 -> Big/64 */
-  if ( srcEndian  == ENDIAN_BIG && hostEndian == ENDIAN_BIG &&
-       srcWidth   == 32         && hostWidth  == 64 )
+  /* Case 2: Big/32 -> Big/64 or Little32 -> little64  */
+  if ( srcEndian  == hostEndian && 
+       srcWidth   == 32 && hostWidth  == 64 )
     {
       A32 *from = (A32 *)src;
       long items = from->n;
@@ -642,7 +692,7 @@ int cvtIfNeeded(void *src, A *dest, I ilen, I cvtInPlace)
 	{
 	  printf("\343 Error:Convert32to64 failed\n");
 	  *dest=NULL;
-	  mf(to);
+	  mf((I*)to);
 	  return -1;
 	}
       else
@@ -665,7 +715,8 @@ int cvtIfNeeded(void *src, A *dest, I ilen, I cvtInPlace)
 
       if( cvtInPlace )
 	{
-	  if(dbg_tb) printf("\343 Converting .m file in-place\n");
+	  static char t[]={"in place"};
+	  if(dbg_tb) beamtrc(t,3,0); /* 3==Converting */
 	  to = (A32 *)src;
 	  to->c=1;		/* set invalid reference count */
 	}
@@ -687,7 +738,7 @@ int cvtIfNeeded(void *src, A *dest, I ilen, I cvtInPlace)
 	  if( cvtInPlace )
 	    printf("\343 File is most likely corrupted-in place conversion\n");
 	  else
-	    mf(to);
+	    mf((I*)to);
 	  
 	  *dest=(A)NULL;
 	  return -1;
@@ -716,7 +767,8 @@ int cvtIfNeeded(void *src, A *dest, I ilen, I cvtInPlace)
 
       if( cvtInPlace )
 	{
-	  if(dbg_tb) printf("\343 Converting .m file in-place\n");
+	  static char t[]={"in place"};
+	  if(dbg_tb) beamtrc(t,3,0); /* 3==Converting */
 	  to = (A64 *)src;
 	  to->c=1;		/* set invalid reference count */
 	}
@@ -738,7 +790,7 @@ int cvtIfNeeded(void *src, A *dest, I ilen, I cvtInPlace)
 	  if( cvtInPlace )
 	    printf("\343 File is most likely corrupted in place conversion\n");
 	  else
-	    mf(to);
+	    mf((I*)to);
 
 	  *dest=(A)NULL;
 	  return -1;
@@ -786,8 +838,8 @@ int cvtIfNeeded(void *src, A *dest, I ilen, I cvtInPlace)
 	{
 	  printf("\343 Error:ConvertEndian32 failed\n");
 	  *dest=NULL;
-	  mf(toa);
-	  mf(tob);
+	  mf((I*)toa);
+	  mf((I*)tob);
 	  return -1;
 	}
       else
@@ -797,13 +849,13 @@ int cvtIfNeeded(void *src, A *dest, I ilen, I cvtInPlace)
 	    {
 	      printf("\343 Error:Convert32to64 failed\n");
 	      *dest=(A)NULL;
-	      mf(toa);
-	      mf(tob);
+	      mf((I*)toa);
+	      mf((I*)tob);
 	      return -1;
 	    }
 	  else
 	    {
-	      mf(toa);
+	      mf((I*)toa);
 	      /* tob->i= (tob->r) ? tob->d[0] : tob->n;  fix items */ 
 	      tob->c=1;		/* set reference count */
 	      *dest=(A)tob;
@@ -864,8 +916,8 @@ int getItems(void *src, I *itemCount, I *rank, I *items, I ilen)
       return 0;
     }
 
-  /* Case 2: Big/32 -> Big/64 */
-  if ( srcEndian  == ENDIAN_BIG && hostEndian == ENDIAN_BIG &&
+  /* Case 2: 32 -> 64 same endian*/
+  if ( srcEndian  == hostEndian &&
        srcWidth   == 32         && hostWidth  == 64 )
     {
       *itemCount = ((A32 *)src)->i;
