@@ -1,12 +1,12 @@
 /*****************************************************************************/
 /*                                                                           */
-/* Copyright (c) 1990-2001 Morgan Stanley Dean Witter & Co. All rights reserved.*/
+/* Copyright (c) 1990-2008 Morgan Stanley All rights reserved.
+*/
 /* See .../src/LICENSE for terms of distribution.                           */
 /*                                                                           */
 /*                                                                           */
 /*****************************************************************************/
 
-
 /*
  * This file replaces b.c to remove all references to the "buddy system"
  * in m.c.  ma(), mab(), and mf() are replaced here with covers to
@@ -25,19 +25,20 @@
 #include <a/k.h>
 #include <a/fir.h>
 #include <a/x.h>
+#include <a/fncdcls.h>
 
-#if (defined(__sgi) && _MIPS_SZLONG == 64) || defined(__sparcv9)
+#if (defined(__sgi) && _MIPS_SZLONG == 64) || defined(__sparcv9) || defined(__ia64) || defined(__x86_64)
 #define MD      63
 #else
 #define MD      31
 #endif
 
-static unsigned long totalSize[MD];
-static unsigned long totalObjs[MD];
-static unsigned long totalSizeMax[MD];
-static unsigned long totalObjsMax[MD];
-static unsigned memTrack=0;	
-static unsigned memTrackThreshold=0;	
+static long totalSize[MD];
+static long totalObjs[MD];
+static long totalSizeMax[MD];
+static long totalObjsMax[MD];
+static long memTrack=0;	
+static long memTrackThreshold=0;	
 
 static pthread_mutex_t memStats_lock = PTHREAD_MUTEX_INITIALIZER;
 
@@ -74,7 +75,7 @@ char * _mab(unsigned long w)
     {
       unsigned long aBytes=w+2*sizeof(*pl);
       printf("\343 ma(): malloc(%lu) failure: errno=%d\n",aBytes,errno);
-      aplus_err(2, aBytes);
+      err(2, (A)aBytes);
       p=(char *)pl;
     }
   else
@@ -131,7 +132,7 @@ char * _mab_cover(unsigned long w)
   return p;
 }
 
-char *(*pf_mab)(unsigned long)=_mab;
+static char *(*pf_mab)(unsigned long)=_mab;
 
 char * mab(unsigned long w) 
 {
@@ -196,6 +197,7 @@ static void _mf_cover(long *p)
 }
 
 void (*pf_mf)(long *)=_mf;
+
 void mf(long *p)
 {
   pf_mf(p);
@@ -247,7 +249,7 @@ static A ep_memStats(A aObj)
     }
   else if(aObj->p[0]==MS(si("info")))
     {
-      int d[9]={MD};
+      I d[9]={MD};
       return (A)gvi(Et,4,
 		    gc(It,1,MD,d,totalSize),
 		    gc(It,1,MD,d,totalObjs),
@@ -264,7 +266,7 @@ static A ep_memStats(A aObj)
 
 void memStatsInstall(void)
 { 
-  install(ep_memStats,"_memStats",A_,1,A_,0,0,0,0,0,0,0);  
+  install((PFI)ep_memStats,"_memStats",A_,1,A_,0,0,0,0,0,0,0);  
   return; 
 }
 
@@ -292,7 +294,7 @@ char * mab(unsigned long w)
     {
       unsigned long aBytes=w+2*sizeof(*pl);
       printf("\343 ma(): malloc(%lu) failure: errno=%d/n",aBytes,errno);
-      aplus_err(2, aBytes);
+      err(2, aBytes);
       p=(char *)pl;
     }
   else
