@@ -1,6 +1,6 @@
 /*****************************************************************************/
 /*                                                                           */
-/* Copyright (c) 1990-2001 Morgan Stanley Dean Witter & Co. All rights reserved.*/
+/* Copyright (c) 1990-2008 Morgan Stanley All rights reserved.*/
 /* See .../src/LICENSE for terms of distribution.                           */
 /*                                                                           */
 /*                                                                           */
@@ -202,7 +202,7 @@ I n;
   CHECKXCP; /* NOTEIT: add  NAN_CHECK and CHECKXCP */
 }
 #else
-NANDD(s_pi,F,F,pif((int)*a,*w))
+NANDD(s_pi,F,F,pif((I)*a,*w))
 #endif
 
 DD(i6,I,I,rm(*w,*a))
@@ -871,7 +871,7 @@ H2(dea){
 #endif
   DO(j--,t[i]=-1);
    {
-    int i, _i;
+    long i, _i;
  
     _i = m;
  
