@@ -1,6 +1,6 @@
 /*****************************************************************************/
 /*                                                                           */
-/* Copyright (c) 1990-2001 Morgan Stanley Dean Witter & Co. All rights reserved.*/
+/* Copyright (c) 1990-2008 Morgan Stanley All rights reserved.*/
 /* See .../src/LICENSE for terms of distribution.                           */
 /*                                                                           */
 /*                                                                           */
@@ -539,7 +539,7 @@ Z I go(void)
     }
     if(rf) 
     {
-      if(s[6]) Reset=MAX(0,MIN(u,atoi(&s[6])));
+      if(s[6]) Reset=MAX(0,MIN(u,atol(&s[6])));
       else Reset=u;
       R 0;
     }  
@@ -697,7 +697,7 @@ I exm(C* expstr,I mode)
  R QE(e)&&XE(e)->f==MN(0)?(ef(e),dc((A)z),(I)aplus_nl):(ef(e),z);
 }
 
-extern I Gf,Xf,doErrorStack;;
+extern I Gf,Xf,doErrorStack;
 I pev(I a)
 {
  I g=G;A z;
