@@ -1,6 +1,6 @@
 /*****************************************************************************/
 /*                                                                           */
-/* Copyright (c) 1990-2001 Morgan Stanley Dean Witter & Co. All rights reserved.*/
+/* Copyright (c) 1990-2008 Morgan Stanley All rights reserved.*/
 /* See .../src/LICENSE for terms of distribution.                           */
 /*                                                                           */
 /*                                                                           */
@@ -12,7 +12,7 @@
 #undef _AIX
 #endif
 #if defined(_AIX)
-extern fsync(),getsockopt(),setsockopt();
+extern closelog(),fsync(),getsockopt(),setsockopt();
 #endif
 #include <stdio.h>
 #if defined(HAVE_SVR4)
@@ -427,7 +427,7 @@ I amsync(A a, A aflags)
     R 0;
   }
   bytes=mf_length(a);
-  R bytes?msync((caddr_t)a, bytes, (int)flags):0;
+  R bytes?msync((caddr_t)a, AH+Tt(a->t,a->n), (int)flags):0;
 #endif /* _AIX */
 }
 
@@ -534,7 +534,7 @@ I mkts1gmt(a)
 #if defined(_AIX) || defined(HAVE_SVR4) || defined (__osf__) || defined(HAVE_MKTIME)
   z->p[0]=mktime(&lt); 
   if( z->p[0] != -1) 
-    z->p[0]=z->p[0]-(long)timezone;  
+    z->p[0]=z->p[0]-timezone;  
 #else
   z->p[0]=timegm(&lt);
 #endif
@@ -687,7 +687,7 @@ A readmat(name)
   {
     start_c = mmap(0, stbuf.st_size, PROT_READ, MAP_SHARED, descriptor, 0);
     (void)close(descriptor);
-    if ((long)start_c == -1) {
+    if ((int)start_c == -1) {
       perror("readmat mmap");
       R (A)gz();
     }
