@@ -3,7 +3,7 @@
 
 /*****************************************************************************/
 /*                                                                           */
-/* Copyright (c) 1990-2001 Morgan Stanley Dean Witter & Co. All rights reserved.*/
+/* Copyright (c) 1990-2008 Morgan Stanley All rights reserved.*/
 /* See .../src/LICENSE for terms of distribution.                           */
 /*                                                                           */
 /*                                                                           */
@@ -13,8 +13,8 @@
 extern "C" {
 #endif
 extern char *ExportAObject(A, char *, I, I *);
-extern int ExportAObjectSizePass(A, const char *,const I, I*, I*);
-extern int ExportAObjectFillPass(A, const char *,const I, I, char *);
+extern I ExportAObjectSizePass(A, const char *,const I, I*, I*);
+extern I ExportAObjectFillPass(A, const char *,const I, I, char *);
 extern A AExportAObject(A, char *, I, I *);
 extern A ImportAObject(char *, I, char *);
 #ifdef __cplusplus
